@@ -12,6 +12,7 @@
 #define _GNU_SOURCE
 
 #include "src/compiled.h"          /* GAP headers                */
+#include "src/iostream.h"          /* Signal Handling            */
 
 #undef PACKAGE
 #undef PACKAGE_BUGREPORT
@@ -23,12 +24,6 @@
 
 #include "pkgconfig.h"    /* our own autoconf results */
 
-/* Note that SIZEOF_VOID_P comes from GAP's config.h whereas
- * SIZEOF_VOID_PP comes from pkgconfig.h! */
-#if SIZEOF_VOID_PP != SIZEOF_VOID_P
-#error GAPs word size is different from ours, 64bit/32bit mismatch
-#endif
-
 #include <stdlib.h>
 #ifdef HAVE_SYS_TYPES_H
 #include <sys/types.h>
@@ -114,12 +109,6 @@ void __stack_chk_fail_local (void)
  *
  * and perhaps:
  *   socketpair, getsockname, poll, setrlimit, getrlimit, getrusage, ulimit,
- * NOTE: There are some problems with respect to signal handling,
- *       because the code for InputOutputLocalProcess and things
- *       has a signal handler for SIGCHLD, which interferes with
- *       things. This is solved in the sense that our SIGCHLD handler
- *       can be switched on and off, thereby providing support for
- *       either InputOutputLocalProcess *or* fork/exec and friends.
  * not for the moment (portability or implementation problems):
  *   remove, scandir, ioctl? (absolutely unportable, as it seems),
  *   fcntl? (for file locking purposes), recvmsg, sendmsg,
@@ -136,14 +125,90 @@ void __stack_chk_fail_local (void)
 
 #define MAXCHLDS 1024
 /* The following arrays make a FIFO structure: */
-static int maxstats = MAXCHLDS;    /* This number must always be the same */
 static int stats[MAXCHLDS];        /* than this number */
 static int pids[MAXCHLDS];         /* and this number! */
 static int fistats = 0;            /* First used entry */
 static int lastats = 0;            /* First unused entry */
 static int statsfull = 0;          /* Flag, whether stats FIFO full */
+
+
+// This function must only be called while IO's signal handler is disabled!
+static int findSignaledPid(int pidc)
+{
+    if (fistats == lastats && !statsfull) /* queue empty */
+        return -1;
+
+    if (pidc == -1)  /* queue not empty and any entry welcome */
+        return fistats;
+
+    int pos = fistats;
+    while (pids[pos] != pidc)
+    {
+        pos++;
+        if (pos >= MAXCHLDS) pos = 0;
+        if (pos == lastats) {
+            pos = -1;  /* None found */
+            break;
+        }
+    }
+    return pos;
+}
+
+// This function must only be called while IO's signal handler is disabled!
+static void removeSignaledPidByPos(int pos)
+{
+    if (fistats == lastats && !statsfull) /* queue empty */
+        return;
+
+    int newpos;
+    if (pos == fistats) {  /* this is the easy case: */
+        fistats++;
+        if (fistats >= MAXCHLDS) fistats = 0;
+    } else {  /* The more difficult case: */
+        do {
+            newpos = pos+1;
+            if (newpos >= MAXCHLDS) newpos = 0;
+            if (newpos == lastats) break;
+            stats[pos] = stats[newpos];
+            pids[pos] = pids[newpos];
+            pos = newpos;
+        } while(1);
+        lastats = pos;
+    }
+    statsfull = 0;
+}
+
+/* This does not have to be the same size as the array above */
+static int ignoredpids[MAXCHLDS];
+static int ignoredpidslen;
+
+static int IO_CheckForIgnoredPid( int pid );
+
 static RETSIGTYPE (*oldhandler)(int whichsig) = 0;  /* the old handler */
 
+static void IO_HandleChildSignal(int retcode, int status)
+{
+   if (retcode > 0) {   /* One of our child processes terminated */
+        if (WIFEXITED(status) || WIFSIGNALED(status)) {
+#ifdef GAP_HasCheckChildStatusChanged
+            if (CheckChildStatusChanged(retcode, status)) {
+                // GAP has dealt with the signal
+            } else
+#endif
+            if (IO_CheckForIgnoredPid(retcode)) {
+                // Previously registered with IO_IgnorePid
+            } else
+            if (!statsfull) {
+                stats[lastats] = status;
+                pids[lastats++] = retcode;
+                if (lastats >= MAXCHLDS) lastats = 0;
+                if (lastats == fistats) statsfull = 1;
+            } else
+                Pr("#E Overflow in table of terminated processes\n",0,0);
+        }
+    }
+}
+
 #ifdef HAVE_SIGNAL
 RETSIGTYPE IO_SIGCHLDHandler( int whichsig )
 {
@@ -152,17 +217,7 @@ RETSIGTYPE IO_SIGCHLDHandler( int whichsig )
      terminated: */
   do {
     retcode = waitpid(-1, &status, WNOHANG);
-    if (retcode > 0) {   /* One of our child processes terminated */
-        if (WIFEXITED(status) || WIFSIGNALED(status)) {
-            if (!statsfull) {
-                stats[lastats] = status;
-                pids[lastats++] = retcode;
-                if (lastats >= maxstats) lastats = 0;
-                if (lastats == fistats) statsfull = 1;
-            } else
-                Pr("#E Overflow in table of terminated processes\n",0,0);
-        }
-    }
+    IO_HandleChildSignal(retcode, status);
   } while (retcode > 0);
 
   signal(SIGCHLD, IO_SIGCHLDHandler);
@@ -191,10 +246,67 @@ Obj FuncIO_RestoreSIGCHLDHandler( Obj self )
   }
 }
 
+// The following function checks if a PID is marked as ignored.
+// Returns 1 if the PID was ignored, 0 otherwise.
+// This function must only be called while IO's signal handler is disabled!
+static int IO_CheckForIgnoredPid( int pid )
+{
+    int i;
+    // Make sure a new signal doesn't come in while looking at array
+    int found = 0;
+    for(i = 0; i < ignoredpidslen; ++i) {
+        if (ignoredpids[i] == pid) {
+            ignoredpids[i] = ignoredpids[ignoredpidslen - 1];
+            ignoredpidslen--;
+            found = 1;
+            break;
+        }
+    }
+    return found;
+}
+
+Obj FuncIO_IgnorePid(Obj self, Obj pid)
+{
+    Int pidc;
+    int pos;
+    if (!IS_INTOBJ(pid)) {
+        return Fail;
+    }
+    pidc = INT_INTOBJ(pid);
+
+    if (pidc < 0) {
+        return Fail;
+    }
+
+    // Make sure a new signal doesn't come in while we are changing array
+    signal(SIGCHLD, SIG_DFL);
+
+    pos = findSignaledPid(pidc);
+    if (pos != -1)
+    {
+        // This PID has already finished
+        removeSignaledPidByPos(pos);
+        signal(SIGCHLD,IO_SIGCHLDHandler);
+        return True;
+    }
+
+    if (ignoredpidslen < MAXCHLDS - 1) {
+        ignoredpids[ignoredpidslen] = pidc;
+        ignoredpidslen++;
+        signal(SIGCHLD,IO_SIGCHLDHandler);
+    }
+    else {
+        Pr("#E Overflow in table of ignored processes",0,0);
+        signal(SIGCHLD,IO_SIGCHLDHandler);
+        return Fail;
+    }
+    return True;
+}
+
 Obj FuncIO_WaitPid(Obj self,Obj pid,Obj wait)
 {
   Int pidc;
-  int pos,newpos;
+  int pos;
   Obj tmp;
   int retcode,status;
   int reallytried;
@@ -204,69 +316,30 @@ Obj FuncIO_WaitPid(Obj self,Obj pid,Obj wait)
   }
   /* First set SIGCHLD to default action to avoid clashes with access: */
   signal(SIGCHLD,SIG_DFL);
+  pidc = INT_INTOBJ(pid);
   reallytried = 0;
   do {
-      pidc = INT_INTOBJ(pid);
-      if (fistats == lastats && !statsfull) /* queue empty */
-          pos = -1;
-      else if (pidc == -1)  /* queue not empty and any entry welcome */
-          pos = fistats;
-      else {  /* Queue nonempty, so look for matching entry: */
-          pos = fistats;
-          do {
-              if (pids[pos] == pidc) break;
-              pos++;
-              if (pos >= maxstats) pos = 0;
-              if (pos == lastats) {
-                  pos = -1;  /* None found */
-                  break;
-              }
-          } while (1);
-      }
-      if (pos != -1) break;  /* we found something! */
+      pos = findSignaledPid(pidc);
+      if (pos != -1)
+          break;  /* we found something! */
       if (reallytried && wait != True) {
           /* Reinstantiate our handler: */
           signal(SIGCHLD,IO_SIGCHLDHandler);
           return False;
       }
-      /* Really wait for something, blocking: */
-      if (wait == True)
-          retcode = waitpid(-1, &status, 0);
-      else
-          retcode = waitpid(-1, &status, WNOHANG);
-      if (retcode > 0) {   /* One of our child processes terminated */
-          if (WIFEXITED(status) || WIFSIGNALED(status)) {
-              /* Append it to the queue: */
-              if (!statsfull) {
-                  stats[lastats] = status;
-                  pids[lastats++] = retcode;
-                  if (lastats >= maxstats) lastats = 0;
-                  if (lastats == fistats) statsfull = 1;
-              } else
-                  Pr("#E Overflow in table of terminated processes\n",0,0);
-          }
-      }
+      /* Really wait for something */
+      retcode = waitpid(-1, &status, (wait == True) ? 0 : WNOHANG);
+      IO_HandleChildSignal(retcode, status);
       reallytried = 1;  /* Do not try again. */
   } while (1);  /* Left by break */
   tmp = NEW_PREC(0);
   AssPRec(tmp,RNamName("pid"),INTOBJ_INT(pids[pos]));
   AssPRec(tmp,RNamName("status"),INTOBJ_INT(stats[pos]));
+  AssPRec(tmp,RNamName("WIFEXITED"), INTOBJ_INT(WIFEXITED(stats[pos])));
+  AssPRec(tmp,RNamName("WEXITSTATUS"), INTOBJ_INT(WEXITSTATUS(stats[pos])));
+
   /* Dequeue element: */
-  if (pos == fistats) {  /* this is the easy case: */
-      fistats++;
-      if (fistats >= maxstats) fistats = 0;
-  } else {  /* The more difficult case: */
-      do {
-          newpos = pos+1;
-          if (newpos >= maxstats) newpos = 0;
-          if (newpos == lastats) break;
-          stats[pos] = stats[newpos];
-          pids[pos] = pids[newpos];
-          pos = newpos;
-      } while(1);
-      lastats = pos;
-  }
-  statsfull = 0;
+  removeSignaledPidByPos(pos);
   /* Reinstantiate our handler: */
   signal(SIGCHLD,IO_SIGCHLDHandler);
   return tmp;
@@ -657,44 +730,11 @@ Obj FuncIO_rmdir(Obj self,Obj path)
 }
 #endif
 
-#ifndef ADDR_INT
-#define ADDR_INT(op)    ((TypDigit*)ADDR_OBJ(op))
-#endif
-
-#ifndef USE_GMP
-static Obj MyObjInt_Int(Int i)
-{
-// FIXME: WTF?
-    Obj n;
-    Int bound = 1L << NR_SMALL_INT_BITS;
-    if (i >= bound) {
-        /* We have to make a big integer */
-        n = NewBag(T_INTPOS,4*sizeof(TypDigit));
-        ADDR_INT(n)[0] = (TypDigit) (i & ((Int) INTBASE - 1L));
-        ADDR_INT(n)[1] = (TypDigit) (i >> NR_DIGIT_BITS);
-        ADDR_INT(n)[2] = 0;
-        ADDR_INT(n)[3] = 0;
-        return n;
-    } else if (-i > bound) {
-        n = NewBag(T_INTNEG,4*sizeof(TypDigit));
-        ADDR_INT(n)[0] = (TypDigit) ((-i) & ((Int) INTBASE - 1L));
-        ADDR_INT(n)[1] = (TypDigit) ((-i) >> NR_DIGIT_BITS);
-        ADDR_INT(n)[2] = 0;
-        ADDR_INT(n)[3] = 0;
-        return n;
-    } else {
-        return INTOBJ_INT(i);
-    }
-}
-#else
-#define MyObjInt_Int(i) ObjInt_Int(i)
-#endif
 
 #ifdef HAVE_STAT
-// FIXME: globals
-static struct stat ourstatbuf;
 Obj FuncIO_stat(Obj self,Obj filename)
 {
+  struct stat ourstatbuf;
   Int res;
   Obj rec;
   if (!IS_STRING(filename) || !IS_STRING_REP(filename)) {
@@ -707,29 +747,28 @@ Obj FuncIO_stat(Obj self,Obj filename)
         return Fail;
       }
       rec = NEW_PREC(0);
-      AssPRec(rec,RNamName("dev"),MyObjInt_Int((Int) ourstatbuf.st_dev));
-      AssPRec(rec,RNamName("ino"),MyObjInt_Int((Int) ourstatbuf.st_ino));
-      AssPRec(rec,RNamName("mode"),MyObjInt_Int((Int) ourstatbuf.st_mode));
-      AssPRec(rec,RNamName("nlink"),MyObjInt_Int((Int) ourstatbuf.st_nlink));
-      AssPRec(rec,RNamName("uid"),MyObjInt_Int((Int) ourstatbuf.st_uid));
-      AssPRec(rec,RNamName("gid"),MyObjInt_Int((Int) ourstatbuf.st_gid));
-      AssPRec(rec,RNamName("rdev"),MyObjInt_Int((Int) ourstatbuf.st_rdev));
-      AssPRec(rec,RNamName("size"),MyObjInt_Int((Int) ourstatbuf.st_size));
-      AssPRec(rec,RNamName("blksize"),MyObjInt_Int((Int)ourstatbuf.st_blksize));
-      AssPRec(rec,RNamName("blocks"),MyObjInt_Int((Int) ourstatbuf.st_blocks));
-      AssPRec(rec,RNamName("atime"),MyObjInt_Int((Int) ourstatbuf.st_atime));
-      AssPRec(rec,RNamName("mtime"),MyObjInt_Int((Int) ourstatbuf.st_mtime));
-      AssPRec(rec,RNamName("ctime"),MyObjInt_Int((Int) ourstatbuf.st_ctime));
+      AssPRec(rec,RNamName("dev"),ObjInt_Int((Int) ourstatbuf.st_dev));
+      AssPRec(rec,RNamName("ino"),ObjInt_Int((Int) ourstatbuf.st_ino));
+      AssPRec(rec,RNamName("mode"),ObjInt_Int((Int) ourstatbuf.st_mode));
+      AssPRec(rec,RNamName("nlink"),ObjInt_Int((Int) ourstatbuf.st_nlink));
+      AssPRec(rec,RNamName("uid"),ObjInt_Int((Int) ourstatbuf.st_uid));
+      AssPRec(rec,RNamName("gid"),ObjInt_Int((Int) ourstatbuf.st_gid));
+      AssPRec(rec,RNamName("rdev"),ObjInt_Int((Int) ourstatbuf.st_rdev));
+      AssPRec(rec,RNamName("size"),ObjInt_Int((Int) ourstatbuf.st_size));
+      AssPRec(rec,RNamName("blksize"),ObjInt_Int((Int)ourstatbuf.st_blksize));
+      AssPRec(rec,RNamName("blocks"),ObjInt_Int((Int) ourstatbuf.st_blocks));
+      AssPRec(rec,RNamName("atime"),ObjInt_Int((Int) ourstatbuf.st_atime));
+      AssPRec(rec,RNamName("mtime"),ObjInt_Int((Int) ourstatbuf.st_mtime));
+      AssPRec(rec,RNamName("ctime"),ObjInt_Int((Int) ourstatbuf.st_ctime));
       return rec;
   }
 }
 #endif
 
 #ifdef HAVE_FSTAT
-// FIXME: globals
-static struct stat ourfstatbuf;
 Obj FuncIO_fstat(Obj self,Obj fd)
 {
+  struct stat ourfstatbuf;
   Int res;
   Obj rec;
   if (!IS_INTOBJ(fd)) {
@@ -742,29 +781,28 @@ Obj FuncIO_fstat(Obj self,Obj fd)
         return Fail;
       }
       rec = NEW_PREC(0);
-      AssPRec(rec,RNamName("dev"),MyObjInt_Int((Int) ourfstatbuf.st_dev));
-      AssPRec(rec,RNamName("ino"),MyObjInt_Int((Int) ourfstatbuf.st_ino));
-      AssPRec(rec,RNamName("mode"),MyObjInt_Int((Int) ourfstatbuf.st_mode));
-      AssPRec(rec,RNamName("nlink"),MyObjInt_Int((Int) ourfstatbuf.st_nlink));
-      AssPRec(rec,RNamName("uid"),MyObjInt_Int((Int) ourfstatbuf.st_uid));
-      AssPRec(rec,RNamName("gid"),MyObjInt_Int((Int) ourfstatbuf.st_gid));
-      AssPRec(rec,RNamName("rdev"),MyObjInt_Int((Int) ourfstatbuf.st_rdev));
-      AssPRec(rec,RNamName("size"),MyObjInt_Int((Int) ourfstatbuf.st_size));
-      AssPRec(rec,RNamName("blksize"),MyObjInt_Int((Int)ourfstatbuf.st_blksize));
-      AssPRec(rec,RNamName("blocks"),MyObjInt_Int((Int) ourfstatbuf.st_blocks));
-      AssPRec(rec,RNamName("atime"),MyObjInt_Int((Int) ourfstatbuf.st_atime));
-      AssPRec(rec,RNamName("mtime"),MyObjInt_Int((Int) ourfstatbuf.st_mtime));
-      AssPRec(rec,RNamName("ctime"),MyObjInt_Int((Int) ourfstatbuf.st_ctime));
+      AssPRec(rec,RNamName("dev"),ObjInt_Int((Int) ourfstatbuf.st_dev));
+      AssPRec(rec,RNamName("ino"),ObjInt_Int((Int) ourfstatbuf.st_ino));
+      AssPRec(rec,RNamName("mode"),ObjInt_Int((Int) ourfstatbuf.st_mode));
+      AssPRec(rec,RNamName("nlink"),ObjInt_Int((Int) ourfstatbuf.st_nlink));
+      AssPRec(rec,RNamName("uid"),ObjInt_Int((Int) ourfstatbuf.st_uid));
+      AssPRec(rec,RNamName("gid"),ObjInt_Int((Int) ourfstatbuf.st_gid));
+      AssPRec(rec,RNamName("rdev"),ObjInt_Int((Int) ourfstatbuf.st_rdev));
+      AssPRec(rec,RNamName("size"),ObjInt_Int((Int) ourfstatbuf.st_size));
+      AssPRec(rec,RNamName("blksize"),ObjInt_Int((Int)ourfstatbuf.st_blksize));
+      AssPRec(rec,RNamName("blocks"),ObjInt_Int((Int) ourfstatbuf.st_blocks));
+      AssPRec(rec,RNamName("atime"),ObjInt_Int((Int) ourfstatbuf.st_atime));
+      AssPRec(rec,RNamName("mtime"),ObjInt_Int((Int) ourfstatbuf.st_mtime));
+      AssPRec(rec,RNamName("ctime"),ObjInt_Int((Int) ourfstatbuf.st_ctime));
       return rec;
   }
 }
 #endif
 
 #ifdef HAVE_LSTAT
-// FIXME: globals
-static struct stat ourlstatbuf;
 Obj FuncIO_lstat(Obj self,Obj filename)
 {
+  struct stat ourlstatbuf;
   Int res;
   Obj rec;
   if (!IS_STRING(filename) || !IS_STRING_REP(filename)) {
@@ -777,19 +815,19 @@ Obj FuncIO_lstat(Obj self,Obj filename)
         return Fail;
       }
       rec = NEW_PREC(0);
-      AssPRec(rec,RNamName("dev"),MyObjInt_Int((Int) ourlstatbuf.st_dev));
-      AssPRec(rec,RNamName("ino"),MyObjInt_Int((Int) ourlstatbuf.st_ino));
-      AssPRec(rec,RNamName("mode"),MyObjInt_Int((Int) ourlstatbuf.st_mode));
-      AssPRec(rec,RNamName("nlink"),MyObjInt_Int((Int) ourlstatbuf.st_nlink));
-      AssPRec(rec,RNamName("uid"),MyObjInt_Int((Int) ourlstatbuf.st_uid));
-      AssPRec(rec,RNamName("gid"),MyObjInt_Int((Int) ourlstatbuf.st_gid));
-      AssPRec(rec,RNamName("rdev"),MyObjInt_Int((Int) ourlstatbuf.st_rdev));
-      AssPRec(rec,RNamName("size"),MyObjInt_Int((Int) ourlstatbuf.st_size));
-      AssPRec(rec,RNamName("blksize"),MyObjInt_Int((Int)ourlstatbuf.st_blksize));
-      AssPRec(rec,RNamName("blocks"),MyObjInt_Int((Int) ourlstatbuf.st_blocks));
-      AssPRec(rec,RNamName("atime"),MyObjInt_Int((Int) ourlstatbuf.st_atime));
-      AssPRec(rec,RNamName("mtime"),MyObjInt_Int((Int) ourlstatbuf.st_mtime));
-      AssPRec(rec,RNamName("ctime"),MyObjInt_Int((Int) ourlstatbuf.st_ctime));
+      AssPRec(rec,RNamName("dev"),ObjInt_Int((Int) ourlstatbuf.st_dev));
+      AssPRec(rec,RNamName("ino"),ObjInt_Int((Int) ourlstatbuf.st_ino));
+      AssPRec(rec,RNamName("mode"),ObjInt_Int((Int) ourlstatbuf.st_mode));
+      AssPRec(rec,RNamName("nlink"),ObjInt_Int((Int) ourlstatbuf.st_nlink));
+      AssPRec(rec,RNamName("uid"),ObjInt_Int((Int) ourlstatbuf.st_uid));
+      AssPRec(rec,RNamName("gid"),ObjInt_Int((Int) ourlstatbuf.st_gid));
+      AssPRec(rec,RNamName("rdev"),ObjInt_Int((Int) ourlstatbuf.st_rdev));
+      AssPRec(rec,RNamName("size"),ObjInt_Int((Int) ourlstatbuf.st_size));
+      AssPRec(rec,RNamName("blksize"),ObjInt_Int((Int)ourlstatbuf.st_blksize));
+      AssPRec(rec,RNamName("blocks"),ObjInt_Int((Int) ourlstatbuf.st_blocks));
+      AssPRec(rec,RNamName("atime"),ObjInt_Int((Int) ourlstatbuf.st_atime));
+      AssPRec(rec,RNamName("mtime"),ObjInt_Int((Int) ourlstatbuf.st_mtime));
+      AssPRec(rec,RNamName("ctime"),ObjInt_Int((Int) ourlstatbuf.st_ctime));
       return rec;
   }
 }
@@ -1483,27 +1521,21 @@ Obj FuncIO_select(Obj self, Obj inlist, Obj outlist, Obj exclist,
 Obj FuncIO_fork(Obj self)
 {
   int res;
+  FuncIO_InstallSIGCHLDHandler(0);
   res = fork();
   if (res == -1) {
       SySetErrorNo();
       return Fail;
   }
-  if (res != 0) {   /* we are the parent */
-      return INTOBJ_INT(res);
-  } else {
-      /* we are the child */
-      return INTOBJ_INT(0);
-  }
+  return INTOBJ_INT(res);
 }
 #endif
 
-// FIXME: globals
-static char *argv[1024];   /* Up to 1024 arguments */
-static char *envp[1024];   /* Up to 1024 environment entries */
-
 Obj FuncIO_execv(Obj self,Obj path,Obj Argv)
 {
     int argc;
+    char *argv[1024];   /* Up to 1024 arguments */
+    char *envp[1024];   /* Up to 1024 environment entries */
     int i;
     Obj tmp;
 
@@ -1535,12 +1567,11 @@ Obj FuncIO_execv(Obj self,Obj path,Obj Argv)
     return Fail;
 }
 
-// FIXME: globals
-extern char **environ;
-
 Obj FuncIO_execvp(Obj self,Obj file,Obj Argv)
 {
     int argc;
+    char *argv[1024];   /* Up to 1024 arguments */
+    char *envp[1024];   /* Up to 1024 environment entries */
     int i;
     Obj tmp;
 
@@ -1575,6 +1606,8 @@ Obj FuncIO_execvp(Obj self,Obj file,Obj Argv)
 Obj FuncIO_execve(Obj self,Obj path,Obj Argv,Obj Envp)
 {
     int argc;
+    char *argv[1024];   /* Up to 1024 arguments */
+    char *envp[1024];   /* Up to 1024 environment entries */
     int i;
     Obj tmp;
 
@@ -1621,6 +1654,8 @@ Obj FuncIO_execve(Obj self,Obj path,Obj Argv,Obj Envp)
     return Fail;
 }
 
+extern char **environ;
+
 Obj FuncIO_environ(Obj self)
 {
     Int i,len;
@@ -1670,15 +1705,6 @@ Obj FuncIO_exit(Obj self,Obj status)
     return True;
 }
 
-Obj FuncIO_MasterPointerNumber(Obj self, Obj o)
-{
-    if ((void **) o >= (void **) MptrBags && (void **) o < (void **) OldBags) {
-        return INTOBJ_INT( ((void **) o - (void **) MptrBags) + 1 );
-    } else {
-        return INTOBJ_INT( 0 );
-    }
-}
-
 #ifdef HAVE_FCNTL_H
 Obj FuncIO_fcntl(Obj self, Obj fd, Obj cmd, Obj arg)
 {
@@ -1734,8 +1760,8 @@ Obj FuncIO_gettimeofday( Obj self )
    struct timeval tv;
    gettimeofday(&tv, NULL);
    tmp = NEW_PREC(0);
-   AssPRec(tmp, RNamName("tv_sec"), MyObjInt_Int( tv.tv_sec ));
-   AssPRec(tmp, RNamName("tv_usec"), MyObjInt_Int( tv.tv_usec ));
+   AssPRec(tmp, RNamName("tv_sec"), ObjInt_Int( tv.tv_sec ));
+   AssPRec(tmp, RNamName("tv_usec"), ObjInt_Int( tv.tv_usec ));
    return tmp;
 }
 #endif
@@ -2012,7 +2038,7 @@ static StructGVarFunc GVarFuncs [] = {
 #ifdef HAVE_MKDTEMP
   { "IO_mkdtemp", 1, "template",
     FuncIO_mkdtemp,
-    "io.c:IO_mkstemp" },
+    "io.c:IO_mkdtemp" },
 #endif
 
 #ifdef HAVE_MKFIFO
@@ -2117,6 +2143,10 @@ static StructGVarFunc GVarFuncs [] = {
     "io.c:IO_select" },
 #endif
 
+  { "IO_IgnorePid", 1, "pid",
+    FuncIO_IgnorePid,
+    "io.c:IO_IgnorePid" },
+
 #if defined(HAVE_SIGACTION) || defined(HAVE_SIGNAL)
   { "IO_WaitPid", 2, "pid, wait",
     FuncIO_WaitPid,
@@ -2163,10 +2193,6 @@ static StructGVarFunc GVarFuncs [] = {
     FuncIO_exit,
     "io.c:IO_exit" },
 
-  { "IO_MasterPointerNumber", 1, "obj",
-    FuncIO_MasterPointerNumber,
-    "io.c:IO_MasterPointerNumber" },
-
 #ifdef HAVE_FCNTL_H
   { "IO_fcntl", 3, "fd, cmd, arg",
     FuncIO_fcntl,
@@ -3108,21 +3134,13 @@ static Int InitLibrary ( StructInitInfo *module )
 */
 static StructInitInfo module = {
 #ifdef IOSTATIC
- /* type        = */ MODULE_STATIC,
+    .type = MODULE_STATIC,
 #else
- /* type        = */ MODULE_DYNAMIC,
+    .type = MODULE_DYNAMIC,
 #endif
- /* name        = */ "io",
- /* revision_c  = */ 0,
- /* revision_h  = */ 0,
- /* version     = */ 0,
- /* crc         = */ 0,
- /* initKernel  = */ InitKernel,
- /* initLibrary = */ InitLibrary,
- /* checkInit   = */ 0,
- /* preSave     = */ 0,
- /* postSave    = */ 0,
- /* postRestore = */ 0
+    .name = "io",
+    .initKernel = InitKernel,
+    .initLibrary = InitLibrary,
 };
 
 #ifndef IOSTATIC
